from typer import Typer, Option, Argument, Exit
from os import mkdir, system
from shutil import copytree
from pathlib import Path
from .config import root_path, themes_path, themes
from rich import print
from .common.dev_server import dev_server

current_path = Path.cwd()

def is_valid_project():
    if not ((current_path / "src").exists() and (current_path / "themes").exists()):
        print("[bold red]Error:[/bold red] Not a valid project directory")
        raise Exit(1)

app = Typer(no_args_is_help=True)

@app.command()
def init(
    project_name: str = Option(..., "--project-name", "-n", help="Name of the project", prompt=True),
    theme: str = Option("BlogX", "--theme", "-t", help="Theme to use", prompt=True),
):
    """Initialize a new project"""
    if theme not in themes:
        # colorful error message
        print(f"[bold red]Error:[/bold red] Theme [bold green]{theme}[/bold green] not found")
        raise Exit(1)

    # Create the project directory
    project_path = Path.cwd() / project_name
    if project_path.exists():
        print(f"[bold red]Error:[/bold red] Project [bold blue]{project_name}[/bold blue] already exists")
        raise Exit(1)
    mkdir(project_path)
    mkdir(project_path / "src")

    # Copy the theme to the project directory
    theme_path = themes_path / theme
    copytree(theme_path, project_path / "themes" / theme)

    # Add index.md and sidebar.md
    with open(project_path / "src" / "index.md", "w", encoding='utf-8') as f:
        f.write("# Welcome to BlogX")
    with open(project_path / "src" / "sidebar.md", "w", encoding='utf-8') as f:
        f.writelines([
            "## Shortcuts\n",
            "- [Home](/)\n",
            "- [Journal](/journal)\n",
            "- [About Me](/about)\n",
            "- [Contact](/contact)\n",
            "## Friends\n",
            "- [Friend 1](https://example.com)\n",
            "- [Friend 2](https://example.com)",
        ])
    print(f"Project [bold red]{project_name}[/bold red] initialized with theme [bold green]{theme}[/bold green]")
    print(f"Run [bold blue]cd {project_name}[/bold blue] to enter the project directory")

@app.command()
def serve():
    """Serve the site"""
    is_valid_project()
    dev_server()

@app.command()
def help(
    command: str = Argument(None, help="Command to get help for"),
):
    if command:
        print(f"Help for {command}")
    else:
        print("Help for the CLI")


from pathlib import Path
from BlogX.common.render import md2html
from functools import lru_cache
from shutil import copytree
import logging

@lru_cache(maxsize=None)
def get_sidebar(sidebar_path: Path):
    with open(sidebar_path, 'r', encoding='utf-8') as f:
        return md2html(f.read())
    
def render(md_content: str, sidebar: str, template: str):
    return template.replace('{{ article_content|safe }}', md2html(md_content)).replace('{{ sidebar_content|safe }}', sidebar)

def build(input_path: Path, output_path: Path, specific_file: Path = None):
    with open(Path(__file__).parent / 'template.html', 'r', encoding='utf-8') as f:
        template = f.read()
    if specific_file:
        if specific_file.suffix == '.md' and specific_file.name != 'sidebar.md':
            logging.info(f"Building {specific_file}")
            specific_file_rel = specific_file.relative_to(input_path)
            output_file = output_path / specific_file_rel.with_suffix('.html')
            with open(specific_file, 'r', encoding='utf-8') as f:
                md_content = f.read()
            with open(output_file, 'w', encoding='utf-8') as f:
                f.write(render(md_content, get_sidebar(input_path / 'sidebar.md'), template))
            return
    logging.info("Building the entire site")
    get_sidebar.cache_clear()
    # copy all files from input_path to output_path
    copytree(input_path, output_path, dirs_exist_ok=True)
    get_sidebar(output_path / 'sidebar.md')
    # convert all .md files to .html files except sidebar.md
    for md_file in output_path.rglob('*.md'):
        if md_file.name != 'sidebar.md':
            html_file = md_file.with_suffix('.html')
            with open(md_file, 'r', encoding='utf-8') as f:
                md_content = f.read()
            with open(html_file, 'w', encoding='utf-8') as f:
                f.write(render(md_content, get_sidebar(output_path / 'sidebar.md'), template))
        md_file.unlink()
    # copy the static folder
    copytree(Path(__file__).parent / 'static', output_path / 'static', dirs_exist_ok=True)


import markdown
import re
from markdown_katex import KatexExtension
from markdown.extensions.codehilite import CodeHiliteExtension

def md2html(md_content):
    # 将$...$替换为$`...`$，$$...$$替换为```math...```，\(...\)替换为$`...`$，\[...]替换为```math...```，以支持 KatexExtension
    md_content = re.sub(r'\$\$(.*?)\$\$', r'```math\1```', md_content, flags=re.DOTALL)  # 块级公式
    md_content = re.sub(r'\$(.*?)\$', r'$`\1`$', md_content)  # 行内公式
    md_content = re.sub(r'\\\((.*?)\\\)', r'$`\1`$', md_content)  # 行内公式
    md_content = re.sub(r'\\\[(.*?)\\\]', r'```math\1```', md_content, flags=re.DOTALL)  # 块级公式
    # 将 Markdown 转换为 HTML，并启用 codehilite、fenced_code、KatexExtension 扩展
    html_content = markdown.markdown(md_content, extensions=[ 
        'fenced_code', 
        'tables', 
        KatexExtension(insert_fonts_css=False), 
        CodeHiliteExtension(linenos=True)])
    # 将所有img标签加上referrerpolicy="no-referrer"属性
    html_content = re.sub(r'<img', r'<img referrerpolicy="no-referrer"', html_content)
    return html_content



from pathlib import Path
from livereload import Server
import importlib.util
import logging
from typer import Exit

current_path = Path.cwd()
theme_path = current_path / "themes" / "BlogX"
builder_path = theme_path / "builder.py"
output_dir = current_path / "dist"

def is_valid_theme(theme_path):
    if not (theme_path / "builder.py").exists():
        print("[bold red]Error:[/bold red] Not a valid theme directory")
        raise Exit(1)

def get_builder(): # load the builder.py file from the theme
    is_valid_theme(theme_path)
    spec = importlib.util.spec_from_file_location("builder", builder_path)
    builder = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(builder)
    return builder

def on_modified(filename: str = "Null"):
    logging.info(f"{filename} modified")
    get_builder().build(current_path / "src", output_dir, Path(filename))

def dev_server():
    get_builder().build(current_path / "src", output_dir)
    # create a server instance
    server = Server()
    # watch the src directory and rebuild the site when a file is modified
    server.watch(current_path / "src", on_modified)
    # start the server
    server.serve(root=output_dir)